(ns clj1365.core
 (:gen-class))

(defn mix-collection-hash-hints
  "Mix final collection hash for ordered or unordered collections.
   hash-basis is the combined collection hash, count is the number
   of elements included in the basis. Note this is the hash code
   consistent with =, different from .hashCode.
   See http://clojure.org/data_structures#hash for full algorithms."
  {:added "1.6"
   :static true}
  ^long [^long hash-basis ^long count] (clojure.lang.Murmur3/mixCollHash hash-basis count))

(defn mix-collection-hash-inline
  "Mix final collection hash for ordered or unordered collections.
   hash-basis is the combined collection hash, count is the number
   of elements included in the basis. Note this is the hash code
   consistent with =, different from .hashCode.
   See http://clojure.org/data_structures#hash for full algorithms."
   {:added "1.6"
    :inline (fn [h c] `(. clojure.lang.Murmur3 (mixCollHash ~h ~c)))
    :inline-arities #{2}
    :static true}
  [hash-basis count] (clojure.lang.Murmur3/mixCollHash hash-basis count))


(set! *unchecked-math* true)
(defn ^long perf-orig [^long n]
  (loop [i 0
         h 0]
    (if (< i n)
      (recur (inc i) (long (mix-collection-hash h i)))
      h)))

(defn ^long perf-hints [^long n]
  (loop [i 0
         h 0]
    (if (< i n)
      (recur (inc i) (long (mix-collection-hash-hints h i))) 
      h)))

(defn ^long perf-inline [^long n]
  (loop [i 0
         h 0]
    (if (< i n)
      (recur (inc i) (long (mix-collection-hash-inline h i)))
      h)))

(defn ^long perf-static [^long n]
  (loop [i 0
         h 0]
    (if (< i n)
      (recur (inc i) (long (clojure.lang.Murmur3/mixCollHash h i)))
      h)))

(defn -main [& args]
  (let [n 1000000
        reps 5]
    (println "mix-collection-hash original")
    (dotimes [_ reps] (time (perf-orig n)))
    (println "mix-collection-hash hints")
    (dotimes [_ reps] (time (perf-hints n)))
    (println "mix-collection-hash inline")
    (dotimes [_ reps] (time (perf-inline n)))
    (println "Murmur3/mixCollHash")
    (dotimes [_ reps] (time (perf-static n)))))
